package com.ruoyi.iot.service.impl;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.utils.NumberUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.iot.constants.ReceiveConstants;
import com.ruoyi.iot.domain.IotDeviceDetail;
import com.ruoyi.iot.domain.ReceiveMsg;
import com.ruoyi.iot.domain.response.CommandResponse;
import com.ruoyi.iot.enums.ReceiveType;
import com.ruoyi.iot.service.IotReceiveService;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.ss.device.domain.Device;
import com.ruoyi.ss.device.domain.enums.DeviceOutageWay;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.transactionBill.service.TransactionAssembler;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;

/**
 * @author wjh
 * 2024/5/6
 */
@Service
@Slf4j
public class IotReceiveServiceImpl implements IotReceiveService {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private TransactionAssembler transactionAssembler;

    @Autowired
    private IotService iotService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Override
    public void handleReceive(ReceiveMsg msg) {
        if (msg == null) {
            return;
        }
        // 数据点推送
        if (ReceiveType.DATA_POINT.getType().equals(msg.getType())) {
            log.info("收到推送数据点：{},{},{}", msg.getAt(), msg.getDsId(), msg.getValue());
            // 若推送数据点:CSQ,则恢复余额
            if (ReceiveConstants.DS_CSQ.equals(msg.getDsId())) {
                this.recoverBalance(msg);
            }
        }
        // 生命周期
        else if (ReceiveType.DEVICE_STATUS.getType().equals(msg.getType())) {
        }
    }

    // 恢复设备的余额
    private void recoverBalance(ReceiveMsg msg) {
        String lockKey = msg.getDevName();
        if (!redisLock.lock(RedisLockKey.RECOVER_DEVICE_BALANCE, lockKey)){
            return;
        }
        try {
            // 查询设备
            DeviceVO device = deviceService.selectByMac(msg.getDevName());
            if (device == null) {
                return;
            }

            // 判断上次恢复余额的时间和本次恢复余额的时间是否相同，若相同则视为重复推送，则忽略
            if (device.getLastRecoverTime() != null && msg.getAt().equals(device.getLastRecoverTime())) {
                return;
            }

            // 待恢复的时长（秒）
            LocalDateTime expireTime = device.getExpireTime();

            // 待恢复的电量（度）
            BigDecimal ele = BigDecimal.ZERO;

            // 若有需要恢复余额的设备，则进行操作
            if (expireTime != null || ele.compareTo(BigDecimal.ZERO) > 0) {
                transactionTemplate.execute(status -> {
                    // 记录上次恢复余额的时间
                    int update = deviceService.updateLastRecoverTime(device.getDeviceId(), msg.getAt());
                    ServiceUtil.assertion(update != 1, "更新设备信息失败");

                    log.info("设备：{} 恢复余额，过期时间：{}，剩余电量：{}", device.getMac(), expireTime, ele);
                    if (expireTime != null ) {
                        int setTime = deviceService.setTime(device, expireTime, true, 3);
                        ServiceUtil.assertion(setTime != 1, "恢复设备时长失败");
                    }
                    if (ele.compareTo(BigDecimal.ZERO) > 0) {
                        CommandResponse res = iotService.trySetEle(device, ele, 3);
                        ServiceUtil.assertion(!res.isSuccess(), "设备电量恢复失败:%s", res.getMsg());
                    }

                    return update;
                });
            }
        } finally {
            redisLock.unlock(RedisLockKey.RECOVER_DEVICE_BALANCE, lockKey);
        }
    }
}
